@@ -114,8 +114,8 @@ html_info (FILE* file,
 	   const struct comment* comm)
 {
   if (html_raw (file, dim) ||
-      html_raw (file, " ") ||
-      html_esc (file, mtime, 0))
+      (*mtime && (html_raw (file, " ") ||
+		  html_esc (file, mtime, 0))))
     return 1;
   for (; comm; comm = comm->next)
     if (html_raw (file, " \n") ||
@@ -318,8 +318,14 @@ html_child_links (FILE* file,
 	html_esc (file, base, 0) ||
 	html_raw (file, "</a>"))
       return 1;
-    if (tree->dirs[i].n_files)
-      fprintf (file, " (%u)", tree->dirs[i].n_files);
+    if (tree->dirs[i].n_files || tree->dirs[i].n_others) {
+      if (tree->dirs[i].n_others)
+	fprintf (file, " (%u+%u)",
+		 tree->dirs[i].n_files,
+		 tree->dirs[i].n_others);
+      else
+	fprintf (file, " (%u)", tree->dirs[i].n_files);
+    }
     if (html_raw (file, "</dt>\n"))
       return 1;
     if (tree->dirs[i].n_dirs) {
@@ -338,12 +344,9 @@ html_child_links (FILE* file,
  * @param level	number of '/' characters in the root directory name
  * @param info	image dimensions
  * @param comm	image comments
+ * @param opts	dump options
  * @param thumb	available thumbnail resolutions
  * @param spec	the thumbnail resolution
- * @param html	index.html file name
- * @param css	index.css file name or URI (optional)
- * @param icon	"icon" file name or URI (optional)
- * @param msico	"shortcut icon" file name or URI (optional)
  * @param top	name of parent directory
  * @param prev	name of previous directory
  * @param next	name of next directory
@@ -355,12 +358,9 @@ dumphtml (FILE* file,
 	  unsigned level,
 	  const struct imageinfo* info,
 	  const struct comment*const* comm,
+	  const struct html_opts* opts,
 	  const struct thumb_spec* thumb,
 	  const struct thumb_spec* spec,
-	  const char* html,
-	  const char* css,
-	  const char* icon,
-	  const char* msico,
 	  const char* top,
 	  const char* prev,
 	  const char* next)
@@ -370,6 +370,8 @@ dumphtml (FILE* file,
   char* fn;
   /** root directory name */
   const char* rootname;
+  /** file modification time */
+  char mtime[20];
   /** length of file name */
   unsigned fnlen = strlen (tree->name) + 1;
   /** length of the thumbnail directory name */
@@ -392,48 +394,49 @@ dumphtml (FILE* file,
       html_esc (file, specstr (spec), 0) ||
       html_raw (file, ")</title>\n"
 		"<meta http-equiv=\"Content-Type\""
-		" content=\"text/html; charset=iso-8859-1\">\n"
-		"<meta name=\"Generator\" content=\"pHoToMoLo 1.1;\n "
+		" content=\"text/html; charset=\"") ||
+      html_raw (file, opts->charset) ||
+      html_raw (file, "\">\n"
+		"<meta name=\"Generator\" content=\"pHoToMoLo 1.2;\n "
 		"http://www.funet.fi/pub/sci/graphics/packages/photomolo/"
 		"photomolo.html\">\n"
 		"<link rel=\"start\" href=\"") ||
-      html_uriref (file, rootname, html) ||
+      html_uriref (file, rootname, opts->html) ||
       html_raw (file, "\">\n"))
     return 1;
-  if (css &&
+  if (opts->css &&
       (html_raw (file, "<link rel=\"stylesheet\" href=\"") ||
-       html_uriref (file, rootname, css) ||
+       html_uriref (file, rootname, opts->css) ||
        html_raw (file, "\">\n")))
     return 1;
-  if (icon &&
+  if (opts->icon &&
       (html_raw (file, "<link rel=\"icon\" href=\"") ||
-       html_uriref (file, rootname, icon) ||
+       html_uriref (file, rootname, opts->icon) ||
        html_raw (file, "\">\n")))
     return 1;
-  if (msico &&
+  if (opts->msico &&
       (html_raw (file, "<link rel=\"SHORTCUT ICON\" href=\"") ||
-       html_uriref (file, rootname, msico) ||
+       html_uriref (file, rootname, opts->msico) ||
        html_raw (file, "\">\n")))
     return 1;
-  if ((top && html_link (file, 0, spec->thumbdir, top, html)) ||
-      (prev && html_link (file, "prev", spec->thumbdir, prev, html)) ||
-      (next && html_link (file, "next", spec->thumbdir, next, html)) ||
-      html_link_specs (file, html, thumb, spec) ||
+  if ((top && html_link (file, 0, spec->thumbdir, top, opts->html)) ||
+      (prev && html_link (file, "prev", spec->thumbdir, prev, opts->html)) ||
+      (next && html_link (file, "next", spec->thumbdir, next, opts->html)) ||
+      html_link_specs (file, opts->html, thumb, spec) ||
       html_raw (file,
 		"</head>\n\n"
 		"<body>\n<h1>") ||
-      html_parent_links (file, rootname, title, html, spec->thumbdir) ||
+      html_parent_links (file, rootname, title, opts->html, spec->thumbdir) ||
       html_esc (file, title, 0) ||
       html_raw (file, "</h1>\n"
 		"<table><tr>\n"
 		"<td>") ||
-      html_child_links (file, tree, html, spec->thumbdir,
+      html_child_links (file, tree, opts->html, spec->thumbdir,
 			strlen (tree->name)) ||
       html_raw (file, "</td>\n"
 		"<td><p>") ||
-      html_a_specs (file, html, thumb, spec) ||
-      html_raw (file, "</p>\n"
-		"<p class=\"img\">"))
+      html_a_specs (file, opts->html, thumb, spec) ||
+      html_raw (file, "</p>\n"))
     return 1;
 
   if (!(fn = malloc (fnlen + speclen + 1))) {
@@ -447,17 +450,22 @@ dumphtml (FILE* file,
   memcpy (fn + fnlen, spec->thumbdir, speclen - 1);
   fn[fnlen - 1] = fn[fnlen + speclen - 1] = '/';
 
-  /* write the descriptions */
+  /* write the image descriptions */
+  if (tree->n_files &&
+      html_raw (file, "<p class=\"img\">")) {
+  error:
+    free (fn);
+    return 1;
+  }
   for (i = 0; i < tree->n_files; i++) {
-    /** file modification time */
-    char mtime[20];
     /** image dimensions */
     char dim[20];
 
     if (!info[i].height || !info[i].width)
       continue; /* skip invalid file */
-    sprintf (dim, "%u\327%u", info[i].width, info[i].height);
-    if (!strftime (mtime, sizeof mtime, "%Y-%m-%d %H:%M:%S",
+    sprintf (dim, "%u&times;%u", info[i].width, info[i].height);
+    if (opts->notim ||
+	!strftime (mtime, sizeof mtime, "%Y-%m-%d %H:%M:%S",
 		   localtime (&tree->files[i].mtime)))
       *mtime = 0;
     if (html_raw (file, "<a href=\"../") ||
@@ -468,11 +476,8 @@ dumphtml (FILE* file,
 	html_info (file, mtime, dim, comm[i]) ||
 	html_raw (file, "]\"\ntitle=\"") ||
 	html_info (file, mtime, dim, comm[i]) ||
-	html_raw (file, "\"")) {
-    error:
-      free (fn);
-      return 1;
-    }
+	html_raw (file, "\""))
+      goto error;
     else {
       /** thumbnail image information */
       struct imageinfo thumb_info;
@@ -505,8 +510,28 @@ dumphtml (FILE* file,
     if (html_raw (file, "></a>\n"))
       goto error;
   }
+  if (tree->n_files && html_raw (file, "</p>\n"))
+    goto error;
+
+  /* list other files */
+  for (i = 0; i < tree->n_others; i++) {
+    if (opts->notim ||
+	!strftime (mtime, sizeof mtime, "%Y-%m-%d %H:%M:%S",
+		   localtime (&tree->others[i].mtime)))
+      *mtime = 0;
+    if (html_raw (file, "<p><a href=\"../") ||
+	html_uri (file, tree->others[i].name) ||
+	(*mtime && (html_raw (file, "\"\ntitle=\"") ||
+		    html_raw (file, mtime))) ||
+	html_raw (file, "\">") ||
+	html_esc (file, tree->others[i].name, 0) ||
+	html_raw (file, "</a></p>\n")) {
+      free (fn);
+      return 1;
+    }
+  }
   free (fn);
-  return html_raw (file, "</p></td>\n"
+  return html_raw (file, "</td>\n"
 		   "</tr></table>"
 		   "</body></html>\n");
 }
