@@ -83,26 +83,30 @@ read_marker (FILE* file)
   return c;
 }
 
-/** Determine whether a character needs to be escaped
- * @param c	the character
+/** Write a character in printable form
+ * @param c	the character to be printed
+ * @param f	the output file
+ * @return	zero on success, nonzero on failure
  */
 static int
-is_printable (int c)
+write_char (int c, FILE* f)
 {
-  return c != '"' && c != '\\' && (c == '\n' || c == '\t' || isprint (c));
+  if (c == '"' || c == '\\')
+    return '\\' != putc ('\\', f) || c != putc (c, f);
+  if (c == '\n' || c == '\t' || isprint (c))
+    return c != putc (c, f);
+  return 4 != fprintf (f, "\\%03o", c);
 }
 
-/** Read image comments
- * @param in	the image file
+/** Read JPEG image comments
+ * @param in	the image file, without the SOI marker
  * @param com	the output file for the comments
  * @return	NULL on success, error message on failure
  */
 static const char*
-readcom (FILE* in,
-	 FILE* com)
+readjpegcom (FILE* in,
+	     FILE* com)
 {
-  if (read_word (in) != 0xffd8)
-    return "not a JPEG image file";
   /* scan markers until start of scan (compressed data) */
   for (;;) {
     unsigned length;
@@ -130,13 +134,31 @@ readcom (FILE* in,
 	  fputs ("\"\n", com);
 	  goto eof;
 	}
-	else if (is_printable (c)
-		 ? (c != putc (c, com))
-		 : (4 != fprintf (com, "\\%03o", c)))
+	else if (write_char (c, com))
 	  goto output;
       }
       if (EOF == fputs ("\"\n", com))
-	goto output;      
+	goto output;
+    }
+    else if ((marker & 0xf0) == 0xc0 &&
+	     marker != 0xc4 && marker != 0xcc) {
+      /* start of frame marker (image dimensions) */
+      /** image dimensions */
+      unsigned height, width;
+      /** number of colour components */
+      int comps;
+      (void) getc (in); /* skip data precision */
+      height = read_word (in);
+      width = read_word (in);
+      comps = getc (in);
+      if (comps == EOF)
+	goto eof;
+      if (length != (6 + ((unsigned) comps) * 3))
+	return "inconsistent SOF marker length";
+      fprintf (com, "# %ux%u\n", width, height);
+      for (length -= 6; length--; )
+	if (getc (in) == EOF)
+	  goto eof;
     }
     else
       while (length--)
@@ -335,13 +357,9 @@ static int
 write_name (const char* name,
 	    FILE* file)
 {
-  for (; *name; *name++) {
-    int c = (unsigned char) *name;
-    if (is_printable (c)
-	? (c != putc (c, file))
-	: (4 != fprintf (file, "\\%03o", c)))
+  for (; *name; *name++)
+    if (write_char ((unsigned char) *name, file))
       return EOF;
-  }
   return 0;
 }
 
@@ -418,7 +436,8 @@ main (int argc, char** argv)
   if (argc > 1) {
     /* read the comments of the listed files */
     int i;
-    if (fputs ("#This is a batch file for"
+    if (argc == 2 &&
+	fputs ("#This is a batch file for"
 	       " commenting, renaming and copying image files.\n"
 	       "#Comment lines start with '#'.\n"
 	       "#Typical usage:"
@@ -446,15 +465,19 @@ main (int argc, char** argv)
 	perror (": open (reading)");
 	return 1;
       }
-      if (fputs ("copy \"", stdout) == EOF ||
-	  write_name (argv[i], stdout) ||
-	  fputs ("\"\t\"", stdout) == EOF ||
-	  write_name (argv[i], stdout) ||
-	  fputs ("\"\n", stdout) == EOF) {
+      if (read_word (f) != 0xffd8) {
+	fputs (argv[i], stderr);
+	fputs (": not a JPEG image file\n", stderr);
+      }
+      else if (fputs ("copy \"", stdout) == EOF ||
+	       write_name (argv[i], stdout) ||
+	       fputs ("\"\t\"", stdout) == EOF ||
+	       write_name (argv[i], stdout) ||
+	       fputs ("\"\n", stdout) == EOF) {
 	fclose (f);
 	goto output;
       }
-      if ((s = readcom (f, stdout))) {
+      else if ((s = readjpegcom (f, stdout))) {
 	fputs (argv[i], stderr);
 	fputs (": ", stderr);
 	fputs (s, stderr);
@@ -535,6 +558,7 @@ main (int argc, char** argv)
 	  if (fd < 0 || !(out = fdopen (fd, "wb"))) {
 	    fputs (name2, stderr);
 	    perror (": open (writing)");
+	  cleanup:
 	    free (name1), free (name2);
 	    return 1;
 	  }
@@ -544,18 +568,25 @@ main (int argc, char** argv)
 	    fputs (r, stderr);
 	    putc ('\n', stderr);
 	    fclose (in), fclose (out);
-	    free (name1), free (name2);
-	    return 1;
+	    goto cleanup;
+	  }
+	  if (fclose (in)) {
+	    fputs (name1, stderr);
+	    perror (": close");
+	    /* ignore failure to close input file */
+	  }
+	  if (fclose (out)) {
+	    fputs (name2, stderr);
+	    perror (": close");
+	    /* abort on failure to close output file */
+	    goto cleanup;
 	  }
-	  fclose (in), fclose (out);
 	  /* copy the timestamp and rename or remove the file */
 	  if (touch_file (name2, name1) ||
 	      (same
 	       ? rename_file (name2, name1)
-	       : (do_remove && remove_file (name1)))) {
-	    free (name1), free (name2);
-	    return 1;
-	  }
+	       : (do_remove && remove_file (name1))))
+	    goto cleanup;
 	  free (name1), free (name2);
 	}
 	else {
