@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <locale.h>
 #include "scale.h"
 #include "tree.h"
 #include "info.h"
@@ -44,18 +45,23 @@ static struct jpeg_compress_struct cout;
 
 /** Thumbnail specifications */
 static struct thumb_spec* thumbs = 0;
-/** Index file name */
-static const char* indexname = "index.html";
-/** Length of the index file name, plus 1 */
+/** libjpeg compression quality */
+static int quality = 75;
+
+/** HTML options */
+static struct html_opts hopts = {
+  0, /**< flag: do not display time stamps for files */
+  "index.html",
+  "index.css",
+  0, /* "icon" file name */
+  0, /* "shortcut icon" file name */
+  "iso-8859-1" /* character set */
+};
+
+/** Length of hopts.html, plus 1 */
 static unsigned indexnamelen;
-/** Style sheet file name or URI */
-static const char* cssname = "index.css";
-/** Length of the style sheet file name, plus 1 */
+/** Length of hopts.css, plus 1 */
 static unsigned cssnamelen;
-/** "icon" file name or URI (optional) */
-static const char* iconname = 0;
-/** "shortcut icon" file name or URI (optional) */
-static const char* shortcutname = 0;
 
 /** flag: generate HTML code? */
 static int dump_html = 1;
@@ -146,17 +152,25 @@ make_thumbnails (const struct dirtree* dir,
 	jpeg_stdio_dest (&cout, output);
 	cout.image_width = spec->max_width;
 	cout.image_height = spec->max_height;
-	if (scale_image (&cin, &cout, spec->shrink) ||
+	if (scale_image (&cin, &cout, spec->shrink, quality) ||
 	    touch_mtime (fn, dir->files[i].mtime)) {
 	  fclose (output);
 	  goto errexit;
 	}
-	fclose (output);
+	if (fclose (output)) {
+	  fputs (fn, stderr);
+	  perror (": close");
+	  goto errexit;
+	}
 	if (spec->next)
 	  rewind (input);
       }
     }
-    fclose (input);
+    if (fclose (input)) {
+      memcpy (fn + fnlen, dir->files[i].name, namelen);
+      fputs (fn, stderr);
+      perror (": close"); /* report but otherwise ignore the error */
+    }
   }
 
   free (fn);
@@ -215,20 +229,20 @@ dumphtmls (const struct dirtree* tree,
       return 2;
     }
     filename[dirnamelen + speclen - 1] = '/';
-    memcpy (filename + dirnamelen + speclen, indexname, indexnamelen);
+    memcpy (filename + dirnamelen + speclen, hopts.html, indexnamelen);
     if (!(f = fopen (filename, "wb"))) {
       fputs (filename, stderr);
       perror (": open");
       free (filename);
       return 1;
     }
-    status = dumphtml (f, tree, level, info, comm, thumbs, spec,
-		       indexname, cssname, iconname, shortcutname,
+    status = dumphtml (f, tree, level, info, comm, &hopts, thumbs, spec,
 		       top, prev, next);
-    fclose (f);
-    if (status) {
+    if (fclose (f) || status) {
       fputs (filename, stderr);
-      perror (": write");
+      perror (status ? ": write" : ": close");
+      if (!status)
+	status = -1;
       break;
     }
   }
@@ -275,8 +289,10 @@ dumpfiles (const struct dirtree* tree,
   filename[dirnamelen++] = '/';
 
   /* gather image info */
-  if (!(info = calloc (tree->n_files, sizeof *info)) ||
-      !(comment = calloc (tree->n_files, sizeof *comment))) {
+  if (!tree->n_files)
+    info = 0, comment = 0;
+  else if (!(info = calloc (tree->n_files, sizeof *info)) ||
+	   !(comment = calloc (tree->n_files, sizeof *comment))) {
     if (info) free (info);
     free (filename);
     goto memory;
@@ -287,11 +303,6 @@ dumpfiles (const struct dirtree* tree,
       unsigned len;
       char* s;
       FILE* f;
-      if (!strcmp (tree->files[i].name, cssname) ||
-	  !strcmp (tree->files[i].name, indexname) ||
-	  (iconname && !strcmp (tree->files[i].name, iconname)) ||
-	  (shortcutname && !strcmp (tree->files[i].name, shortcutname)))
-	continue; /* ignore known non-image file names */
       len = strlen (tree->files[i].name) + 1;
       if (!(s = realloc (filename, dirnamelen + len))) {
 	fputs ("out of memory\n", stderr);
@@ -332,8 +343,8 @@ dumpfiles (const struct dirtree* tree,
   /* deallocate the image comments */
   for (i = tree->n_files; i--; )
     freeinfo (comment[i]);
-  free (info);
-  free (comment);
+  if (tree->n_files)
+    free (info), free (comment);
   return status;
 }
 
@@ -416,7 +427,7 @@ create_css (const char* path)
   FILE* f;
   unsigned pathlen;
   char* s;
-  if (*cssname == '/' || strstr (cssname, "://"))
+  if (*hopts.css == '/' || strstr (hopts.css, "://"))
     return 0; /* absolute location */
   pathlen = strlen (path);
   if (!(s = malloc (pathlen + 1 + cssnamelen))) {
@@ -426,7 +437,7 @@ create_css (const char* path)
   }
   memcpy (s, path, pathlen);
   s[pathlen++] = '/';
-  memcpy (s + pathlen, cssname, cssnamelen);
+  memcpy (s + pathlen, hopts.css, cssnamelen);
   if ((f = fopen (s, "rt")))
     goto done;
   if (!(f = fopen (s, "wt"))) {
@@ -461,10 +472,11 @@ main (int argc, char** argv)
   jpeg_create_compress (&cout);
 
   atexit (cleanup);
+  setlocale (LC_ALL, "");
 
   if (argc == 1) {
   Usage:
-    fputs ("pHoToMoLo 1.1\n"
+    fputs ("pHoToMoLo 1.2\n"
 	   "Create thumbnails and HTML navigation\n"
 	   "Usage: photomolo [options] directory1 directory2 ...\n"
 	   "Options:\n"
@@ -473,16 +485,23 @@ main (int argc, char** argv)
 	   "-T <dir>/<spec>\n"
 	   "\tgenerate thumbnail directory <dir> according to <spec>\n"
 	   "-H htmlfile\n"
-	   "\tname of HTML files (default is \"index.html\")\n"
+	   "\tname of HTML files (default=\"index.html\")\n"
 	   "-C cssfile\n"
-	   "\tname of CSS files (default is \"index.css\")\n"
+	   "\tname of CSS files (default=\"index.css\")\n",
+	   stderr);
+    fputs ("-c charset\n"
+	   "\tHTML and JPEG comment character set (default=\"iso-8859-1\")\n"
 	   "-i iconfile\n"
 	   "-I file.ico\n"
-	   "\tname of icon files (default is none)\n"
+	   "\tname of icon files (default=none)\n"
 	   "-t"
 	   "\tgenerate thumbnails only (no HTML or CSS)\n"
 	   "-r"
 	   "\tremove the thumbnail directories listed with -T\n"
+	   "-q quality\n"
+	   "\tJPEG compression quality (default=75)\n"
+	   "-d"
+	   "\tomit timestamps from the HTML\n"
 	   "--\tdisable processing further options\n",
 	   stderr);
     return 0;
@@ -529,49 +548,57 @@ main (int argc, char** argv)
 	break;
       case 'H':
 	if (argc <= 2) {
-	  fputs ("-H: missing argument (use -h for help\n", stderr);
+	  fputs ("-H: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
-	if (!*(indexname = *++argv)) {
+	if (!*(hopts.html = *++argv)) {
 	  fputs ("-H: argument must be nonempty (use -h for help)\n", stderr);
 	  return 1;
 	}
-	else if (strchr (indexname, '/')) {
+	else if (strchr (hopts.html, '/')) {
 	  fputs ("-H ", stderr);
-	  fputs (indexname, stderr);
+	  fputs (hopts.html, stderr);
 	  fputs (": name must not contain '/'", stderr);
 	  return 1;
 	}
 	break;
       case 'C':
 	if (argc <= 2) {
-	  fputs ("-C: missing argument (use -h for help\n", stderr);
+	  fputs ("-C: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
-	if (!*(cssname = *++argv)) {
+	if (!*(hopts.css = *++argv)) {
 	  fputs ("-C: argument must be nonempty (use -h for help)\n", stderr);
 	  return 1;
 	}
 	break;
+      case 'c':
+	if (argc <= 2) {
+	  fputs ("-c: missing argument (use -h for help)\n", stderr);
+	  return 1;
+	}
+	argc--;
+	hopts.charset = *++argv; /* no sanity check */
+	break;
       case 'i':
 	if (argc <= 2) {
 	  fputs ("-i: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
-	if (!*(iconname = *++argv))
-	  iconname = 0;
+	if (!*(hopts.icon = *++argv))
+	  hopts.icon = 0;
 	break;
       case 'I':
 	if (argc <= 2) {
-	  fputs ("-I: missing argument (use -h for help\n", stderr);
+	  fputs ("-I: missing argument (use -h for help)\n", stderr);
 	  return 1;
 	}
 	argc--;
-	if (!*(shortcutname = *++argv))
-	  shortcutname = 0;
+	if (!*(hopts.msico = *++argv))
+	  hopts.msico = 0;
 	break;
       case 't':
 	dump_html = 0;
@@ -579,6 +606,24 @@ main (int argc, char** argv)
       case 'r':
 	remove_thumbdirs = 1;
 	break;
+      case 'd':
+	hopts.notim = 1;
+	break;
+      case 'q':
+	if (argc <= 2) {
+	  fputs ("-q: missing argument (use -h for help)\n", stderr);
+	  return 1;
+	}
+	else {
+	  char* endp;
+	  unsigned long q = strtoul (*++argv, &endp, 0);
+	  argc--;
+	  if (q < 1L || q > 100L || *endp || !*argv) {
+	    fputs ("-q: quality must be between 1 and 100\n", stderr);
+	    return 1;
+	  }
+	  quality = q;
+	}
       }
     }
   }
@@ -588,8 +633,8 @@ main (int argc, char** argv)
     return 1;
   }
 
-  indexnamelen = strlen (indexname) + 1;
-  cssnamelen = strlen (cssname) + 1;
+  indexnamelen = strlen (hopts.html) + 1;
+  cssnamelen = strlen (hopts.css) + 1;
   for (; argc > 1; argv++, argc--) {
     struct dirtree dirs;
     /* number of slashes in the specified directory */
@@ -613,7 +658,8 @@ main (int argc, char** argv)
     }
     inittree (&dirs, *argv);
     if (readtree (&dirs, prune_dirs) ||
-	dumptree (&dirs, num_slash, 0, 0, 0)) {
+	(freefileinfo (&dirs.n_others, &dirs.others),
+	 dumptree (&dirs, num_slash, 0, 0, 0))) {
       freetree (&dirs);
       return 2;
     }
